@@ -1,12 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from collections import defaultdict
+
 from trytond.pool import PoolMeta, Pool
 from trytond.model import fields
 from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
 
-__all__ = ['Statement', 'StatementLine']
-
 
 class Statement(metaclass=PoolMeta):
     __name__ = 'account.statement'
@@ -19,8 +19,8 @@ class Statement(metaclass=PoolMeta):
 
         moves = super(Statement, cls).create_move(statements)
 
-        to_success = set()
-        to_fail = set()
+        to_success = defaultdict(set)
+        to_fail = defaultdict(set)
         for move, statement, lines in moves:
             for line in lines:
                 if line.payment:
@@ -32,15 +32,19 @@ class Statement(metaclass=PoolMeta):
                 else:
                     continue
                 if (kind == 'receivable') == (line.amount >= 0):
-                    to_success.update(payments)
+                    to_success[line.date].update(payments)
                 else:
-                    to_fail.update(payments)
+                    to_fail[line.date].update(payments)
         # The failing should be done last because success is usually not a
         # definitive state
         if to_success:
-            Payment.succeed(Payment.browse(to_success))
+            for date, payments in to_success.items():
+                with Transaction().set_context(clearing_date=date):
+                    Payment.succeed(Payment.browse(payments))
         if to_fail:
-            Payment.fail(Payment.browse(to_fail))
+            for date, payments in to_fail.items():
+                with Transaction().set_context(clearing_date=date):
+                    Payment.fail(Payment.browse(payments))
 
         for move, statement, lines in moves:
             assert len({l.payment for l in lines}) == 1
@@ -106,7 +110,8 @@ class StatementLine(metaclass=PoolMeta):
         default.setdefault('payment_group', None)
         return super(StatementLine, cls).copy(lines, default=default)
 
-    @fields.depends('payment', 'party', 'account', '_parent_statement.journal')
+    @fields.depends('payment', 'party', 'account',
+        'statement', '_parent_statement.journal')
     def on_change_payment(self):
         pool = Pool()
         Currency = pool.get('currency.currency')
@@ -114,7 +119,7 @@ class StatementLine(metaclass=PoolMeta):
             if not self.party:
                 self.party = self.payment.party
             clearing_account = self.payment.journal.clearing_account
-            if not self.account and clearing_account:
+            if clearing_account and self.account != clearing_account:
                 self.account = clearing_account
             if self.statement and self.statement.journal:
                 with Transaction().set_context(date=self.payment.date):
@@ -129,7 +134,7 @@ class StatementLine(metaclass=PoolMeta):
         if self.payment_group:
             self.party = None
             clearing_account = self.payment_group.journal.clearing_account
-            if not self.account and clearing_account:
+            if clearing_account and self.account != clearing_account:
                 self.account = clearing_account
 
     @fields.depends('party', 'payment')
@@ -161,3 +166,36 @@ class StatementLine(metaclass=PoolMeta):
                 if l.payment
                 and l.payment.clearing_move
                 and l.payment.clearing_move.state == 'draft'])
+
+
+class StatementRuleLine(metaclass=PoolMeta):
+    __name__ = 'account.statement.rule.line'
+
+    def get_line(self, origin, keywords, **context):
+        line = super().get_line(origin, keywords, **context)
+        if line:
+            line.payment = self._get_payment(origin, keywords)
+            if (line.payment and line.party
+                    and line.payment.party != line.party):
+                return
+            line.payment_group = self._get_payment_group(origin, keywords)
+        return line
+
+    def _get_payment(self, origin, keywords):
+        pool = Pool()
+        Payment = pool.get('account.payment')
+        if keywords.get('payment'):
+            payments = Payment.search([('rec_name', '=', keywords['payment'])])
+            if len(payments) == 1:
+                payment, = payments
+                return payment
+
+    def _get_payment_group(self, origin, keywords):
+        pool = Pool()
+        Payment = pool.get('account.payment.group')
+        if keywords.get('payment_group'):
+            groups, = Payment.search(
+                [('rec_name', '=', keywords['payment_group'])])
+            if len(groups) == 1:
+                group, = groups
+                return group
